//+------------------------------------------------------------------+
//|                                          Flexi_bars_swing_v2.mq4 |
//|                                                         Zen_Leow |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Zen_Leow"
#property link      ""

#property indicator_chart_window

// The number of buffers for calculation, up to 8
#property indicator_buffers 2

// The color for displaying arrows
#property indicator_color1 Green       // Long signal
#property indicator_color2 Red         // Short signal

// Width of the arrows
#property indicator_width1 2  // Long signal arrow
#property indicator_width2 2  // Short signal arrow

extern string TimeFrame        = "Current time frame";
extern int ArrowDistance = 3;
extern bool WaitForCandleClose = false;
extern bool ArrowOnTriggerBar = false;

// Buffers for the calculations
double Up_Arrow_Buffer[];    // Long buffer for display
double Down_Arrow_Buffer[];   // Short buffer for display
double Right_Side_Buffer[];
int RightSideCandleNum = 0;
int SignalIndex = 0;
int ArrowOffset = 0;
string indicatorFileName;
int    timeFrame;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if (WaitForCandleClose)
   {
      SignalIndex = 1;
   }
   else
   {
      SignalIndex = 0;
   }
   // SetIndexStyle  - sets the new type, style, width and color for a given indicator line
   // SetIndexBuffer - binds the array variable declared at a global level to the custom indicator pre-defined buffer
   // SetIndexArrow  - sets an arrow symbol for indicators line of the DRAW_ARROW type.
   IndicatorBuffers(3);
   SetIndexStyle(0, DRAW_ARROW);
   SetIndexBuffer(0, Up_Arrow_Buffer);
   SetIndexArrow(0, 233); // Up arrow
   SetIndexStyle(1, DRAW_ARROW);
   SetIndexBuffer(1, Down_Arrow_Buffer);
   SetIndexArrow(1, 234); // Down arrow
   SetIndexBuffer(2, Right_Side_Buffer);
         indicatorFileName = WindowExpertName();
         timeFrame         = stringToTimeFrame(TimeFrame);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if (timeFrame!=Period())
   {
      int limit = MathMax(MathMin(Bars-IndicatorCounted(),Bars-1),MathMin(6*timeFrame/Period(),Bars-1));
   
       for(int i = limit; i >= 0; i--)
       {
         int y = iBarShift(NULL,timeFrame,Time[i]);
         int x = iBarShift(NULL,timeFrame,Time[i+1]);
            if (x!=y)
            {
               Up_Arrow_Buffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",ArrowDistance,WaitForCandleClose,ArrowOnTriggerBar,0,y);
               Down_Arrow_Buffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"",ArrowDistance,WaitForCandleClose,ArrowOnTriggerBar,1,y);
            }
            else
            {
               Up_Arrow_Buffer[i]   = EMPTY_VALUE;
               Down_Arrow_Buffer[i] = EMPTY_VALUE;
            }
       }
       return(0);
   }

   int Counted_bars;                // Number of counted bars
   //--------------------------------------------------------------------   
   Counted_bars=IndicatorCounted(); // Number of counted bars   
   i=Bars-Counted_bars-1;           // Index of the first uncounted
   if (i == 0)
   {
      i = 5;  // the newest signal bar is suppose to be at index 2 or 3
   }
   while(i>SignalIndex)            // Loop for uncounted bars but always recount the newest 3 bars
   {      
      if (isUpSwingBar(i))
      {
         if (ArrowOnTriggerBar)
         {
            ArrowOffset = RightSideCandleNum;
         }
         else
         {
            ArrowOffset = 0;
         }
         if (RightSideCandleNum == 1 || (RightSideCandleNum == 2 && i > SignalIndex+1))
         {
            Up_Arrow_Buffer[i-ArrowOffset] = Low[i-ArrowOffset] - (ArrowDistance * Point);
            Down_Arrow_Buffer[i-ArrowOffset] = EMPTY_VALUE;
            Right_Side_Buffer[i] = RightSideCandleNum;
         }
      }
      else if (isDownSwingBar(i))
      {
         if (ArrowOnTriggerBar)
         {
            ArrowOffset = RightSideCandleNum;
         }
         else
         {
            ArrowOffset = 0;
         }
         if (RightSideCandleNum == 1 || (RightSideCandleNum == 2 && i > SignalIndex+1))
         {
            Down_Arrow_Buffer[i-ArrowOffset] = High[i-ArrowOffset] + (ArrowDistance * Point);
            Up_Arrow_Buffer[i-ArrowOffset] = EMPTY_VALUE;
            Right_Side_Buffer[i] = RightSideCandleNum;
         }
      }
      else
      {
         if (ArrowOnTriggerBar)
         {
            ArrowOffset = Right_Side_Buffer[i];
         }
         else
         {
            ArrowOffset = 0;
         }
         Up_Arrow_Buffer[i-ArrowOffset] = EMPTY_VALUE;
         Down_Arrow_Buffer[i-ArrowOffset] = EMPTY_VALUE;
         Right_Side_Buffer[i] = EMPTY_VALUE;
        // RightSideCandleNum = 0;
      }
      i--;
   }
//----
   return(0);
}

bool isUpSwingBar(int i)
{
   if (UpSwingLeftOk(i) && UpSwingRightOk(i))
   {
      return (true);
   }
   return (false);
}

bool UpSwingLeftOk(int i)
{
   if (Low[i] < Low[i+1])
   {
      if (High[i] < High[i+1])
      {
         return (true);
      }
      else
      {
         if (Low[i] < Low[i+2] && High[i] < High[i+2])
         {
            return (true);
         }
      }
   }
   return (false);
}

bool UpSwingRightOk(int i)
{
   if (Low[i] < Low[i-1])
   {
      if (High[i] < High[i-1])
      {
         RightSideCandleNum = 1;
         return (true);
      }
      else
      {
         if (Low[i] < Low[i-2] && High[i] < High[i-2])
         {
            RightSideCandleNum = 2;
            return (true);
         }
      }
   }
   return (false);
}

bool isDownSwingBar(int i)
{
   if (DownSwingLeftOk(i) && DownSwingRightOk(i))
   {
      return (true);
   }
   return (false);
}

bool DownSwingLeftOk(int i)
{
   if (High[i] > High[i+1])
   {
      if (Low[i] > Low[i+1])
      {
         return (true);
      }
      else
      {
         if (Low[i] > Low[i+2] && High[i] > High[i+2])
         {
            return (true);
         }
      }
   }
   return (false);
}

bool DownSwingRightOk(int i)
{
   if (High[i] > High[i-1])
   {
      if (Low[i] > Low[i-1])
      {
         RightSideCandleNum = 1;
         return (true);
      }
      else
      {
         if (Low[i] > Low[i-2] && High[i] > High[i-2])
         {
            RightSideCandleNum = 2;
            return (true);
         }
      }
   }
   return (false);
}
//+------------------------------------------------------------------+

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}